/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        17.10.2008
 Description:    MyScope

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "config.h"
#include "usart.h"


//----------------------------------------------------------------------------
//Hier startet das Hauptprogramm
int main(void)
{    
    unsigned char tmp = 0;
    usart_init(BAUDRATE);
    usart_write("MYSCOPE compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
    
    usart_status.usart_ready = 1;
    
    sei();


    DDRB |= (1<<PB0)|(1<<PB1);
    
    //Power Led On
    PORTB |= (1<<PB0);

    //AC/DC Messung (1=DC)
    PORTB |= (1<<PB1);
    
    DDRA = 0x00;
	//PORTA = 0xff;
	
    DDRC = (1<<RES)|(1<<RUN)|(1<<CLK)|(1<<DIV0)|(1<<DIV1); 
    PORTC = 0x00;  

    //Set TIMER2 (PWM OC2 Pin) Scope Offset Spannung erzeugen
	DDRD |= (1<<7);
	TCCR2 |= (1<<WGM21|1<<WGM20|1<<COM21|1<<CS20);
	OCR2 = 170;

    while(1)
    {
 
        //*************************************************************************   
        //Reset des Adresscounters
        PORTC |= (1<<RES);
        PORTC &= ~(1<<RES); 
          
        //Beschreiben des RAMs
        PORTC |= (1<<RUN);
		
        while (PINC & (1<<STO));
		
        PORTC &= ~(1<<RUN);
        
        //************************************************************************* 
        if (usart_status.usart_ready == 1)
        { 

            OCR2 = usart_rx_buffer[0] + usart_rx_buffer[1];  
  
            //AC oder DC
            if (usart_rx_buffer[4] == 0)
            {
                //AC/DC Messung (1=DC)
                PORTB |= (1<<PB1);
            }
            else
            {
                //AC/DC Messung (1=DC)
                PORTB &= ~(1<<PB1);
            }
            
            if (usart_rx_buffer[3] == 0)
            {
                //Scope Time Div CPLD CLK/0
                PORTC &= ~(1<<DIV0);
                PORTC &= ~(1<<DIV1);
            }
            if (usart_rx_buffer[3] == 1)
            {
                //Scope Time Div CPLD CLK/2/32/256
                PORTC |= (1<<DIV0);
                PORTC &= ~(1<<DIV1);
            }
            if (usart_rx_buffer[3] == 2)
            {
                //Scope Time Div CPLD CLK/2/32/256
                PORTC |= (1<<DIV1);
                PORTC &= ~(1<<DIV0);
            }
            if (usart_rx_buffer[3] == 3)
            {
                //Scope Time Div CPLD CLK/2/32/256
                PORTC |= (1<<DIV0);
                PORTC |= (1<<DIV1);
            }
                  
            //Reset des Adresscounters
            PORTC |= (1<<RES);
            PORTC &= ~(1<<RES); 
                
            usart_status.usart_ready = 0;    
            
            //Read RAM 
            DDRA = 0x00; //PortA als Eingang zum lesen der Daten
            //while (PINC & (1<<STO))
            
            //Trigger-Level finden
			unsigned char trigger_level = 128;
            unsigned char trigger_typ = 1;
            
            unsigned int trigger_counter;  
            
			for(trigger_counter = 0; trigger_counter < 8000;trigger_counter++)
            {  
                //RAM Adresscounter +1
                PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
                asm("nop");
				tmp = PINA;
                PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
                if (trigger_typ)
                {
                    if (tmp > (trigger_level + 20))
                    {
                        break;
                    }
                }
                else
                {
                    if (tmp < (trigger_level - 20))
                    {
                        break;
                    }
                }
            }
            trigger_counter = 8000 - trigger_counter;
            
            for(unsigned int a = 0; a < 8000 + trigger_counter;a++)
            {  
                //RAM Adresscounter +1
                PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
                asm("nop");
				tmp = PINA;
                PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
                if (trigger_typ)
                {
                    if (tmp <= trigger_level)
                    {
                        break;
                    }
                }
                else
                {
                    if (tmp >= trigger_level)
                    {
                        break;
                    }
                }
            }
            
            //Daten zum PC bertragen
            for(unsigned int a = 0; a < 400;a++)
            {  
                //RAM Adresscounter +1
                PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
				asm("nop");
				tmp = PINA;
                usart_write_char(tmp);
                PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
               
                unsigned int loop = 0;
                if (usart_rx_buffer[2] > 0)
                {
                    loop = (1<<(usart_rx_buffer[2]-1));
                }
                while(loop--)
                {
                    PORTC |= (1<<CLK); //Clock Adresscounter +1 (High)
                    PORTC &= ~(1<<CLK);//Clock Adresscounter (LOW)
                }
                
            }  
        }
    
    }
}

